package io.quarkiverse.openapi.generator.deployment.codegen;

import static io.quarkiverse.openapi.generator.deployment.SpecConfig.API_PKG_SUFFIX;
import static io.quarkiverse.openapi.generator.deployment.SpecConfig.MODEL_PKG_SUFFIX;
import static io.quarkiverse.openapi.generator.deployment.SpecConfig.getResolvedBasePackageProperty;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.stream.Stream;

import io.quarkiverse.openapi.generator.deployment.wrapper.OpenApiClientGeneratorWrapper;
import io.quarkus.bootstrap.prebuild.CodeGenException;
import io.quarkus.deployment.CodeGenContext;
import io.quarkus.deployment.CodeGenProvider;

/**
 * Code generation for OpenApi Client. Generates Java classes from OpenApi spec files located in src/main/openapi or
 * src/test/openapi
 * <p>
 * Wraps the <a href="https://openapi-generator.tech/docs/generators/java">OpenAPI Generator Client for Java</a>
 */
public abstract class OpenApiGeneratorCodeGenBase implements CodeGenProvider {

    static final String YAML = ".yaml";
    static final String YML = ".yml";
    static final String JSON = ".json";

    @Override
    public String inputDirectory() {
        return "openapi";
    }

    @Override
    public boolean trigger(CodeGenContext context) throws CodeGenException {
        final Path outDir = context.outDir();
        final Path openApiDir = context.inputDir();

        if (Files.isDirectory(openApiDir)) {
            try (Stream<Path> openApiFilesPaths = Files.walk(openApiDir)) {
                openApiFilesPaths
                        .filter(Files::isRegularFile)
                        .map(Path::toString)
                        .filter(s -> s.endsWith(this.inputExtension()))
                        .map(this::encodeURISpaces)
                        .forEach(openApiFilePath -> {
                            final String basePackage = getRequiredIndexedProperty(
                                    getResolvedBasePackageProperty(openApiFilePath), context);
                            final OpenApiClientGeneratorWrapper generator = new OpenApiClientGeneratorWrapper(
                                    "file:" + openApiFilePath, outDir.toString())
                                            .withApiPackage(basePackage + API_PKG_SUFFIX)
                                            .withModelPackage(basePackage + MODEL_PKG_SUFFIX);
                            generator.generate();
                        });
            } catch (IOException e) {
                throw new CodeGenException("Failed to generate java files from OpenApi files in " + openApiDir.toAbsolutePath(),
                        e);
            }
            return true;
        }
        return false;
    }

    private String encodeURISpaces(String path) {
        // the underlying swagger parser library handles the path as a URI, and in previous versions (the one used by the openapi-generator)
        // it has a bug, not handling these paths correctly. That's why we manually encode the paths here, so the file can be accessible and validated against
        // these libraries
        return path.replaceAll(" ", "%20");
    }

    private String getRequiredIndexedProperty(final String propertyKey, final CodeGenContext context) {
        // this is how we get a required property. The configSource will handle the exception for us.
        return context.config().getValue(propertyKey, String.class);
    }
}
